@@ -476,11 +476,24 @@ static void learn(VCC *vcc,unsigned long ip,struct sockaddr_atmsvc *addr)
 	if (vcc) {
 	    entry = vcc->entry;
 	    if (!entry->itf) Q_REMOVE(unknown_incoming,entry);
+	    else if (entry->ip && entry->ip != ip && (entry->flags & ATF_PERM))
+		  {
+		    diag(COMPONENT,DIAG_ERROR,"ignoring attempt to change IP "
+		      "address of permanent entry (to %d.%d.%d.%d)",ipp[0],
+		      ipp[1],ipp[2],ipp[3]);
+		    return;
+		}
 	}
 	else {
 	    entry = alloc_entry(1);
 	    entry->flags = ATF_PUBL;
 	}
+    if (entry->addr && (entry->flags & ATF_PERM) &&
+      !atm_equal(entry->addr,addr,0,0)) {
+	diag(COMPONENT,DIAG_ERROR,"ignoring attempt to change ATM address of "
+	  "permanent entry");
+	return;
+    }
     if (entry->state == as_valid && entry->ip == ip && entry->addr &&
       atm_equal(entry->addr,addr,0,0)) return; /* no news */
     STOP_TIMER(entry);
