@@ -32,9 +32,19 @@
  * and we would like to express our thanks.
  */
 
+/* Change Log:
+ * 1-30-97 Added VERIFY state to allow resending of GetNext
+ *         PDUs after a ColdStart.  This addresses a bug that
+ *         some switches have with receiving the GetNext too
+ *         soon after the ColdStart.  Also added state IDLE
+ *         for a more configurable poll period.
+ */
+
 #include <unistd.h>
 #include <sys/time.h>
 #include <errno.h>
+#include "asn_incl.h"
+#include "rfc1155_smi.h"
 #include "rfc1157_snmp.h"
 #include "message.h"
 #include "util.h"
@@ -43,25 +53,28 @@
 #include "mib.h"
 #include "netprefix.h"
 
-#define WAIT_TIMEOUT 5
+#define RESPONSE_TIMEOUT 2
+#define POLL_PERIOD 15
 #define COMPONENT "ILMI"
 
-typedef enum IlmiState { NO_CHANGE, RESTART, NET_PREFIX, ADDRESS, POLL } IlmiState;
-
+typedef enum IlmiState { NO_CHANGE, RESTART, VERIFY, NET_PREFIX,
+                         ADDRESS, POLL, IDLE } IlmiState;
+const char *state2text[7] = { "NO_CHANGE", "RESTART", "VERIFY", "NET_PREFIX",
+                              "ADDRESS", "POLL", "IDLE" };
 void usage(char *name);
 void ilmi_loop(int fd, int itf);
 
 void usage(char *name)
 {
-  fprintf(stderr, "usage: %s [-b] [-d] [-l logfile] [-x] [-q qos] "
+  fprintf(stderr, "usage: %s [-b] [-d] [-v] [-l logfile] [-x] [-q qos] "
     "<interface>\n", name);
   exit(1);
 }
 
 void ilmi_loop(int fd, int itf)
 {
-  int poll_misses = 0;
-  long int requestID = 0;
+  int no_response;
+  long int requestID;
   Message *poll_message, *set_message, *coldstart_message, *in_message;
   AsnOid *set_oid, *netprefix_oid, *esi_oid;
   VarBind *varbind;
@@ -80,34 +93,43 @@ void ilmi_loop(int fd, int itf)
   for(;;)
     {
       state = new_state;
+      diag(COMPONENT, DIAG_DEBUG, "entering state %s", state2text[state]);
+
       /* Output for the current state */
       switch(state)
 	{
 	case RESTART:
 	  deleteNetPrefix();
-	  diag(COMPONENT, DIAG_DEBUG, "sending cold-start");
-	  diag(COMPONENT, DIAG_DEBUG, "sending get-next");
-	  send_message(fd, coldstart_message);
-	  sleep(1); /* @@@@@ HACK */
+	  diag(COMPONENT, DIAG_INFO, "sending cold-start");
+          send_message(fd, coldstart_message);
+          no_response = 0;
+
+        case VERIFY:
+	  diag(COMPONENT, DIAG_INFO, "sending get-next");
 	  poll_message->data->a.get_next_request->request_id = ++requestID;
 	  send_message(fd, poll_message);
 	  break;
 
 	case ADDRESS:
-	  diag(COMPONENT, DIAG_DEBUG, "setting the atm address on the switch");
+	  diag(COMPONENT, DIAG_INFO, "setting the atm address on the switch");
 	  set_message->data->a.set_request->request_id = ++requestID;
 	  send_message(fd, set_message);
 	  break;
 
 	case POLL:
-	  diag(COMPONENT, DIAG_DEBUG, "sending get-next request");
+	  diag(COMPONENT, DIAG_INFO, "sending get-next request");
 	  poll_message->data->a.get_next_request->request_id = ++requestID;
 	  send_message(fd, poll_message);
 	  break;
 	}
 
-      timeout.tv_sec = WAIT_TIMEOUT;
+      /* Set the time-out period */
+      if(state == IDLE)
+        timeout.tv_sec = POLL_PERIOD;
+      else
+        timeout.tv_sec = RESPONSE_TIMEOUT;
       timeout.tv_usec = 0;
+
       new_state = NO_CHANGE;
       /* Input handling loop */
       while(new_state == NO_CHANGE)
@@ -119,7 +141,7 @@ void ilmi_loop(int fd, int itf)
 		switch(in_message->data->choiceId)
 		  {
 		  case PDUS_GET_REQUEST:
-		    diag(COMPONENT, DIAG_DEBUG, "received get request");
+		    diag(COMPONENT, DIAG_INFO, "received get request");
 		    MIBget(in_message->data->a.get_next_request->variable_bindings,
 			   &in_message->data->a.get_next_request->error_status,
 			   &in_message->data->a.get_next_request->error_index);
@@ -128,7 +150,7 @@ void ilmi_loop(int fd, int itf)
 		    break;
 		    
 		  case PDUS_GET_NEXT_REQUEST:
-		    diag(COMPONENT, DIAG_DEBUG, "received get-next request");
+		    diag(COMPONENT, DIAG_INFO, "received get-next request");
 		    MIBgetnext(in_message->data->a.get_next_request->variable_bindings,
 			       &in_message->data->a.get_next_request->error_status,
 			       &in_message->data->a.get_next_request->error_index);
@@ -137,13 +159,14 @@ void ilmi_loop(int fd, int itf)
 		    break;
 		    
 		  case PDUS_GET_RESPONSE:
-		    diag(COMPONENT, DIAG_DEBUG, "received get response");
+		    diag(COMPONENT, DIAG_INFO, "received get response");
 		    if(in_message->data->a.get_response->request_id == requestID)
 		      {
 			varbind = (VarBind *) FIRST_LIST_ELMT(in_message->data->a.get_response->variable_bindings);
 			switch(state)
 			  {
 			  case RESTART:
+                          case VERIFY:
 			    if(in_message->data->a.get_response->error_status == NOERROR &&
 			       AsnOidMember(&varbind->name, &atmAddressStatus))
 			      new_state = RESTART;
@@ -154,9 +177,9 @@ void ilmi_loop(int fd, int itf)
 			  case ADDRESS:
 			    if(in_message->data->a.get_response->error_status == NOERROR)
 			      {
-				diag(COMPONENT, DIAG_INFO, "ATM address registered");
+				diag(COMPONENT, DIAG_WARN, "ATM address registered");
 				update_nsap(itf, netprefix_oid, esi_oid);
-				poll_misses = 0;
+				no_response = 0;
 				new_state = POLL;
 			      }
 			    else
@@ -166,7 +189,10 @@ void ilmi_loop(int fd, int itf)
 			  case POLL:
 			    if(in_message->data->a.get_response->error_status == NOERROR &&
 			       AsnOidEqual(&varbind->name, set_oid))
-			      poll_misses = 0;
+                            {
+			       no_response = 0;
+                               new_state = IDLE;
+                            }
 			    else
 			      new_state = RESTART;
 			    break;
@@ -176,7 +202,7 @@ void ilmi_loop(int fd, int itf)
 		    break;
 		    
 		  case PDUS_SET_REQUEST:
-		    diag(COMPONENT, DIAG_DEBUG, "received set request");
+		    diag(COMPONENT, DIAG_INFO, "received set request");
 		    MIBset(in_message->data->a.set_request->variable_bindings,
 			   &in_message->data->a.set_request->error_status,
 			   &in_message->data->a.set_request->error_index);
@@ -185,7 +211,7 @@ void ilmi_loop(int fd, int itf)
 		    break;
 		    
 		  case PDUS_TRAP:
-		    diag(COMPONENT, DIAG_DEBUG, "received trap");
+		    diag(COMPONENT, DIAG_INFO, "received trap");
 		    if(in_message->data->a.trap->generic_trap == COLDSTART &&
 		       state != RESTART)
 		      new_state = RESTART;
@@ -197,8 +223,8 @@ void ilmi_loop(int fd, int itf)
 		if((state == NET_PREFIX || new_state == NET_PREFIX) &&
 		   (netprefix_oid = accessNetPrefix()) != NULL)
 		  {
-		    diag(COMPONENT, DIAG_DEBUG, "switch registered a network prefix");
-		    set_oid->octetLen = ADDRESS_LEN + 2;
+		    diag(COMPONENT, DIAG_INFO, "switch registered a network prefix");
+		    set_oid->octetLen = ADDRESS_LEN + 1;
 		    AsnOidAppend(set_oid, netprefix_oid);
 		    AsnOidAppend(set_oid, esi_oid);
 		    new_state = ADDRESS;
@@ -209,26 +235,33 @@ void ilmi_loop(int fd, int itf)
 	      switch(state)
 		{
 		case RESTART:
-		  diag(COMPONENT, DIAG_DEBUG, "switch did not respond to get-next -- resending");
-		  new_state = RESTART;
+                case VERIFY:
+		  diag(COMPONENT, DIAG_INFO, "switch did not respond to get-next -- resending");
+                  if(++no_response == 4)
+                    new_state = RESTART;
+                  else
+		    new_state = VERIFY;
 		  break;
 		case NET_PREFIX:
-		  diag(COMPONENT, DIAG_DEBUG, "switch did not register a network prefix -- restarting");
+		  diag(COMPONENT, DIAG_INFO, "switch did not register a network prefix -- restarting");
 		  new_state = RESTART;
 		  break;
 		case ADDRESS:
-		  diag(COMPONENT, DIAG_DEBUG, "switch did not respond to set request -- resending");
+		  diag(COMPONENT, DIAG_INFO, "switch did not respond to set request -- resending");
 		  new_state = ADDRESS;
 		  break;		  
 		case POLL:
-		  if(++poll_misses == 4)
+		  if(++no_response == 4)
 		    {
 		      new_state = RESTART;
-		      diag(COMPONENT, DIAG_DEBUG, "switch is not responding");
+		      diag(COMPONENT, DIAG_INFO, "switch is not responding");
 		    }
 		  else
-		    new_state = POLL;
+		    new_state = IDLE;
 		  break;
+                case IDLE:
+                  new_state = POLL;
+                  break;
 		}
 	    }
 	}
@@ -242,14 +275,17 @@ int main(int argc, char *argv[])
   const char *qos;
 
   set_application("ilmid");
-  set_verbosity(NULL, DIAG_INFO);
+  set_verbosity(NULL, DIAG_WARN);
   qos = NULL;
-  while((opt = getopt(argc, argv, "bdhl:q:x")) != EOF)
+  while((opt = getopt(argc, argv, "bdhl:q:xv")) != EOF)
     switch (opt)
       {
       case 'd':
-	set_verbosity(NULL, DIAG_DEBUG);
+	set_verbosity(NULL, DIAG_INFO);
 	break;
+      case 'v':
+        set_verbosity(NULL, DIAG_DEBUG);
+        break;
       case 'b':
 	bg = 1;
 	break;
@@ -273,7 +309,7 @@ int main(int argc, char *argv[])
   if(argc != optind)
     usage(argv[0]);
 
-  diag(COMPONENT, DIAG_INFO, "Linux ATM ILMI, version %s", VERSION);
+  diag(COMPONENT, DIAG_WARN, "Linux ATM ILMI, version %s", VERSION);
 
   if(bg)
     {
