@@ -36,6 +36,8 @@
 #define IO_H
 
 #include <sys/time.h>
+#include "asn_incl.h"
+#include "rfc1155_smi.h"
 #include "rfc1157_snmp.h"
 
 #define MAX_ILMI_MSG 484
