@@ -558,6 +558,7 @@ conn_call_callbacks(void)
   Sap_client_t *sap;
   
   conn = connlist;
+  EVENT(EM_DEBUG,("Conn_call_callbacks %p\n",connlist));
   while(conn) {
     if (conn->status == CONNECTED) {
       /* Connection made, not notified yet */
@@ -579,6 +580,8 @@ conn_call_callbacks(void)
 			     0);
       conn = conn->next;
     } else if (conn->status == RELEASED) {
+      EVENT(EM_DEBUG,("Conn_call_callbacks, released conn %p\n",conn));
+#if 0
       conn->status = NOTIFIED;
       sap = (Sap_client_t *)conn->sap_handle;
       sap->vc_notify_callback(conn->conn_context,
@@ -595,6 +598,17 @@ conn_call_callbacks(void)
 	mem_free(EINST, conn->conn_info);
       mem_free(EINST,conn);
       conn=c_tmp;
+#endif
+      conn->status = NOTIFIED;
+      c_tmp = conn->next;
+      sap = (Sap_client_t *)conn->sap_handle;
+      sap->vc_notify_callback(conn->conn_context,
+			      conn,
+			      CONN_RELEASE,
+			      CAUSE_NORMAL,
+			      0,
+			      (conn->type==WEMADE)?TRUE:FALSE);
+      conn=c_tmp;
     } else {
       conn = conn->next;
     }
@@ -806,6 +820,7 @@ conn_check_incoming(fd_set *fds)
   }
   conn=connlist;
   while (conn) {
+    tmpconn = conn->next;
     if (FD_ISSET(conn->fd, fds)) {
       EVENT(EM_DEBUG,("Event in fd:%d\n",conn->fd));
       if (conn->type == KERNEL_SOCK) { /* Message from kernel */
@@ -870,7 +885,7 @@ conn_check_incoming(fd_set *fds)
 	}
       }
     }
-    conn = conn->next;
+    conn = tmpconn;
   }
   return reset;
 }
