@@ -61,7 +61,7 @@ int main(int argc,const char *const *argv)
 	    argv++;
 	    argc--;
 	}
-    if ((s = socket(PF_ATMPVC,SOCK_DGRAM,ATM_AAL5)) < 0) {
+    if ((s = socket(PF_ATMPVC,SOCK_DGRAM,0)) < 0) {
 	perror("socket");
 	return 1;
     }
