@@ -1,6 +1,6 @@
 /* zntune.c - ZN122x free buffer pool tuning */
  
-/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
+/* Written 1995-1997 by Werner Almesberger, EPFL-LRC */
  
 
 #include <stdlib.h>
@@ -10,9 +10,8 @@
 #include <sys/socket.h>
 #include <sys/ioctl.h>
 #include <linux/atm.h>
-
 #include <sys/time.h> /* for struct timeval, although it's not used */
-#include "/usr/src/linux/drivers/atm/zatm.h"
+#include <linux/atm_zatm.h>
 
 
 static void usage(const char *name)
