@@ -1,6 +1,6 @@
 /* third.c - Phase III, code generation */
 
-/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
+/* Written 1995-1997 by Werner Almesberger, EPFL-LRC */
 
 
 #include <stdlib.h>
@@ -29,6 +29,10 @@ static void construct(FIELD *start)
     int patch_here;
 
     for (walk = start; walk; walk = walk->next) {
+	if (walk->structure) {
+	    construct(walk->structure->block);
+	    continue;
+	}
 	if (!walk->value)
 	    if (walk->var_len == -1)
 		code("%s%d%d%d/* %s */\n","OP_COPY",walk->jump,walk->pos,
@@ -101,6 +105,10 @@ static void parser(FIELD *start,int level,int group)
     BREAK *brk,*tmp_brks,*next;
 
     for (walk = start; walk; walk = walk->next) {
+	if (walk->structure) {
+	    parser(walk->structure->block,level,group);
+	    continue;
+	}
 	if (walk->brk) {
 	    code("%s%s\n","OP_IFEND","?");
 	    brk = alloc_t(BREAK);
