@@ -255,8 +255,6 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 		    if (sock->state == ss_connected)
 			diag(COMPONENT,DIAG_INFO,"Active close (CR 0x%06X)",
 			  sock->call_ref);
-		    /* fall through */
-		case ss_hold:
 		    send_release(sock,
 #ifdef UNI31
 		      ATM_CV_NORMAL_CLEAR
@@ -265,8 +263,7 @@ static void dispatch(SOCKET *sock,struct atmsvc_msg *msg)
 #endif
 		      );
 		    START_TIMER(sock,T308_1);
-		    new_state(sock,sock->state != ss_hold ? ss_rel_req :
-		      ss_wait_rel);
+		    new_state(sock,ss_rel_req);
 		    return;
 		case ss_rel_ind:
 		    send_release_complete(sock->call_ref,0); /* @@@ */
