@@ -1,6 +1,6 @@
 /* read.c - receive AAL5 PDU */
 
-/* Written 1995,1996 by Werner Almesberger, EPFL-LRC */
+/* Written 1995-1997 by Werner Almesberger, EPFL-LRC */
 
 
 #include <stdlib.h>
@@ -31,7 +31,7 @@ int main(int argc,char **argv)
     int s;
 
     if (argc != 2) usage(argv[0]);
-    if ((s = socket(PF_ATMPVC,SOCK_DGRAM,ATM_AAL5)) < 0) {
+    if ((s = socket(PF_ATMPVC,SOCK_DGRAM,0)) < 0) {
 	perror("socket");
 	return 1;
     }
@@ -39,6 +39,7 @@ int main(int argc,char **argv)
     if (text2atm(argv[1],(struct sockaddr *) &addr,sizeof(addr),
       T2A_PVC | T2A_UNSPEC | T2A_WILDCARD) < 0) usage(argv[0]);
     memset(&qos,0,sizeof(qos));
+    qos.aal = ATM_AAL5;
     qos.rxtp.traffic_class = ATM_UBR;
     qos.rxtp.max_sdu = BSIZE;
     if (setsockopt(s,SOL_ATM,SO_ATMQOS,&qos,sizeof(qos)) < 0) {
